@@ -16,6 +16,7 @@
 #include <ft2build.h>
 #include FT_FREETYPE_H
 #include FT_SFNT_NAMES_H
+#include FT_TRUETYPE_IDS_H
 #include FT_TYPE1_TABLES_H
 #include <cairo.h>
 #include <cairo-pdf.h>
@@ -34,6 +35,7 @@
 #include <math.h>
 #include <libintl.h>
 #include <locale.h>
+#include <iconv.h>
 
 #include "unicode_blocks.h"
 #include "config.h"
@@ -64,6 +66,7 @@ static struct option longopts[] = {
   {"style", 1, 0, 't'},
   {"font-index", 1, 0, 'n'},
   {"other-index", 1, 0, 'm'},
+  {"no-embed", 0, 0, 'e'},
   {0, 0, 0, 0}
 };
 
@@ -80,6 +83,7 @@ static const char *output_file_name;
 static bool postscript_output;
 static bool svg_output;
 static bool print_outline;
+static bool no_embed;
 static struct range *ranges;
 static struct range *last_range;
 static int font_index;
@@ -298,7 +302,7 @@ static void parse_options(int argc, char * const argv[])
 	for (;;) {
 		int c;
 
-		c = getopt_long(argc, argv, "f:o:hd:sgli:x:t:n:m:", longopts, NULL);
+		c = getopt_long(argc, argv, "f:o:hd:sgli:x:t:n:m:e", longopts, NULL);
 
 		if (c == -1)
 			break;
@@ -357,6 +361,9 @@ static void parse_options(int argc, char * const argv[])
 		case 'm':
 			other_index = atoi(optarg);
 			break;
+		case 'e':
+			no_embed = true;
+			break;
 		case '?':
 		default:
 			usage(argv[0]);
@@ -633,7 +640,14 @@ static int draw_unicode_block(cairo_t *cr, cairo_scaled_font_t *font,
 		}
 
 		/* Show previously positioned glyphs */
-		cairo_show_glyphs(cr, glyphs, nglyphs);
+		if (no_embed) {
+			cairo_save(cr);
+			cairo_glyph_path (cr, glyphs, nglyphs);
+			cairo_fill(cr);
+			cairo_restore(cr);
+		} else {
+			cairo_show_glyphs(cr, glyphs, nglyphs);
+		}
 
 		for (i = 0; i < tbl_end - tbl_start; i++)
 			if (filled_cells[i])
@@ -697,6 +711,7 @@ static void usage(const char *cmd)
 			"  --postscript-output, -s              Use PostScript format for output instead of PDF\n"
 			"  --svg,               -g              Use SVG format for output\n"
 			"  --print-outline,     -l              Print document outlines data to standard output\n"
+			"  --no-embed,          -e              Don't embed the font in the output file, draw the glyphs instead\n"
 			"  --include-range,     -i RANGE        Show characters in RANGE\n"
 			"  --exclude-range,     -x RANGE        Do not show characters in RANGE\n"
 			"  --style,             -t \"STYLE: VAL\" Set STYLE to value VAL\n"));
@@ -713,21 +728,33 @@ static void usage(const char *cmd)
 static const char *get_font_name(FT_Face face)
 {
 	FT_Error error;
-	FT_SfntName face_name;
-	char *fontname;
+	char *fontname = NULL;
 
 	/* try SFNT format */
-	error = FT_Get_Sfnt_Name(face, 4 /* full font name */, &face_name);
-	if (!error) {
-		fontname = malloc(face_name.string_len + 1);
-		if (!fontname) {
-			perror("malloc");
-			exit(1);
+	unsigned num_names = FT_Get_Sfnt_Name_Count(face);
+	iconv_t u16to8 = iconv_open("UTF-8", "UTF-16BE");
+	for (unsigned i = 0; i < num_names; i++) {
+		FT_SfntName name;
+
+		error = FT_Get_Sfnt_Name(face, i, &name);
+		if (error)
+			continue;
+
+		if (name.name_id == TT_NAME_ID_FULL_NAME &&
+		    name.platform_id == TT_PLATFORM_MICROSOFT &&
+		    name.encoding_id == TT_MS_ID_UNICODE_CS) {
+			fontname = malloc(name.string_len * 2 + 1);
+			char *bufptr = fontname;
+			size_t inbytes = name.string_len;
+			size_t outbytes = name.string_len * 2;
+			if (iconv(u16to8, (char**)&name.string, &inbytes, &bufptr, &outbytes) == (size_t)-1)
+				continue;
+			*bufptr = '\0';
 		}
-		memcpy(fontname, face_name.string, face_name.string_len);
-		fontname[face_name.string_len] = '\0';
-		return fontname;
 	}
+	iconv_close(u16to8);
+	if (fontname)
+		return fontname;
 
 	/* try Type1 format */
 	PS_FontInfoRec fontinfo;
